"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ace_1 = require("@adonisjs/ace");
const utils_1 = require("../src/utils");
const path_1 = require("path");
const http_1 = __importDefault(require("http"));
const url_1 = __importDefault(require("url"));
const debug = require('debug')('MailBuilder_');
function requireUncached(module) {
    delete require.cache[require.resolve(module)];
    return require(module);
}
/**
 * Launch queue workers to start processing
 *
 * @version 2.0.0
 * @adonis-version 5.0+
 */
class MailBuilderServe extends ace_1.BaseCommand {
    constructor(app, kernel) {
        super(app, kernel);
        this.mailbuilderFolderPath = utils_1.getMailBuilderCompiledClassesPath(this.application);
        this.mailBuilder = app.container.use('Adonis/Addons/MailBuilder');
        this.config = app.container.use('Adonis/Core/Config');
    }
    /**
     * Execute command
     */
    async handle() {
        const PORT = this.config.get('mailbuilder.devPort') || '3335';
        const URL = `http://localhost:${PORT}`;
        if (!(await this.hasFiles())) {
            this.logger.error('MailBuilders is not exists, please make your first mailbuilder use command: "node ace make:mailbuilder"');
            return;
        }
        // const mailbuilderTmpPath = await getMailBuilderTmpPath()
        http_1.default
            .createServer(async (req, res) => {
            let currUrl = url_1.default.parse(req.url || '').pathname || '';
            currUrl = currUrl.replace('/', '');
            const files = (await utils_1.listFiles(this.mailbuilderFolderPath)).filter((f) => f.includes('.js') && !f.includes('.map'));
            // console.log(currUrl)
            if (files.includes(currUrl)) {
                res.writeHead(200, {
                    'Content-Type': 'text/html',
                });
                try {
                    // console.log('currUrl', this.mailbuilderFolderPath, currUrl)
                    const pathToFile = path_1.join(this.mailbuilderFolderPath, currUrl);
                    const MailBuilderClass = new (requireUncached(pathToFile).default)();
                    const { html } = await this.mailBuilder.render(MailBuilderClass);
                    return res.end(html + utils_1.livereloadScript(URL, currUrl));
                }
                catch (e) {
                    debug(e);
                    return res.end(e.message + utils_1.livereloadScript(URL, currUrl));
                }
            }
            else if (currUrl === '') {
                const mappedFiles = files.map((f) => `<a href="${URL}/${f}"><h4>${f}</h4>`);
                return res.end(utils_1.indexPageRender(mappedFiles.join('\n') + utils_1.livereloadScript(URL, '')));
            }
            else {
                res.statusCode = 404;
                return res.end(`Path is not finded`);
            }
        })
            .listen(PORT);
        this.logger.success(`Test server is running: ${URL}`);
        // prevent the main process from exiting...
        await new Promise(() => () => { });
    }
    async hasFiles() {
        try {
            let mailbuilderFiles = await utils_1.listFiles(this.mailbuilderFolderPath);
            return mailbuilderFiles && mailbuilderFiles.length ? true : false;
        }
        catch (e) {
            return false;
        }
    }
}
exports.default = MailBuilderServe;
MailBuilderServe.commandName = 'mailbuilder:serve';
MailBuilderServe.description = 'Start one or more workers';
MailBuilderServe.settings = {
    loadApp: true,
};
